@@ -93,7 +93,7 @@ MSGS= {
          'ANS':['id', 'ok'],
          'ADD':['id', 'list', 'number', 'passport1', 'passport2'],
          'REM':['id', 'list', 'number', 'passport'],
-         'BYE':['arg'],
+         'BYE':['arg', 'idleclose'],
          'REA':['id', 'listNum', 'passport', 'name'],
          'PRP':['type', 'phone'],
          'LST':['passport', 'nick', 'lists', 'groups'],
@@ -102,6 +102,7 @@ MSGS= {
          'CCL':[],
          'FIL':['size'],
          'QNG':[],
+         'NOT':['size'],
        }
 
 ERRORS= {      
@@ -548,6 +549,9 @@ class Session( Comm ):
         self.lstNum= 0
         self.lstTotal= 0
 
+        # For hiding socket error when server closes idle chats
+        self.hideSocketError= 0
+
     def connect( self, user, passwd, state ):
         self.user= user
         self.passwd= passwd
@@ -559,7 +563,7 @@ class Session( Comm ):
         self.ready= 1
         #self.s.setblocking( 0 )
 
-        self.send( 'VER', 'MSNP8 CVRO\n' )
+        self.send( 'VER', 'MSNP8 CVRO\r\n' )
 
     def reconnect( self ):
         self.inputs.remove( self.s )
@@ -601,7 +605,7 @@ class Session( Comm ):
         self.pendingChats[str(self.id)]= chat
         if passport:
             chat.addUser( passport )
-        self.send( 'XFR', 'SB\n' )
+        self.send( 'XFR', 'SB\r\n' )
 
         return chat
 
@@ -675,7 +679,7 @@ class Session( Comm ):
         
         self.chats[chat.s]= chat
         self.pendingChats[str(self.id)]= chat
-        self.send( 'XFR', 'SB\n' )
+        self.send( 'XFR', 'SB\r\n' )
         chat.connected= None
     
     def closeChat( self, chat ):
@@ -689,7 +693,7 @@ class Session( Comm ):
         for fRecv in chat.recvFile.values():
             self.removeFileRecv( fRecv )
 
-        chat.send( 'OUT', '\n' )
+        chat.send( 'OUT', '\r\n' )
         del( self.chats[chat.s] )
         self.delConnectingChat( chat )
         self.inputs.remove( chat.s )
@@ -708,17 +712,23 @@ class Session( Comm ):
                 #return chat[1]
 
     def changeState( self, state ):
-        self.send( 'CHG', '%s\n' % state )
+        ## Added to allow Pebrot to stay alive offline
+	if state == "FLN":
+            self.send ( 'OUT', '\r\n' )
+            self.state = 'FLN'
+            self.pInfo( 'You have been logged out. Type \\q to exit or \\s nln to reconnect.' )
+	else:
+	    self.send( 'CHG', '%s\r\n' % state )
 
     def challenge( self, msg ):
         chunk= md5.new( msg.fields['hash']+ 'Q1P7W2E4J9R8U3S5' ).hexdigest()
-        self.send( 'QRY', 'msmsgs@msnmsgr.com 32\n%s' % chunk )
+        self.send( 'QRY', 'msmsgs@msnmsgr.com 32\r\n%s' % chunk )
 
     def ping( self ):
         self.sendRaw( 'PNG\r\n' )
 
     def requestInfo( self, infoId= 0 ):
-        self.send( 'SYN', '%d\n' % infoId );
+        self.send( 'SYN', '%d\r\n' % infoId );
     
     def addUser( self, passport, list ):    
         '''Add user to list.
@@ -730,11 +740,11 @@ class Session( Comm ):
                 RL: reverse list'''
 
         self.listOps[int(self.id)]= (passport, list, 'ADD')
-        self.send( 'ADD', ' %s %s %s\n' % (list, passport, passport) )
+        self.send( 'ADD', ' %s %s %s\r\n' % (list, passport, passport) )
 
     def delUser( self, passport, list ):    
         self.listOps[int(self.id)]= (passport, list, 'REM')
-        self.send( 'REM', ' %s %s\n' % (list, passport) )
+        self.send( 'REM', ' %s %s\r\n' % (list, passport) )
 
     def acceptChat( self, msg ):    
         chat= Chat( self )
@@ -751,7 +761,7 @@ class Session( Comm ):
         name= name[:DISPLAYNAME_MAX]
         name= unicode( name, ENCODING, 'replace' )
         name= urllib.quote(name.encode( 'utf-8' ))
-        self.send( 'REA', ' %s %s\n' % (self.user, name) )
+        self.send( 'REA', ' %s %s\r\n' % (self.user, name) )
      
     def onlyOnline( self, x ):
         #print >>sys.stderr, x, 'esta a', self.usersOnline.has_key( x )
@@ -774,6 +784,9 @@ class Session( Comm ):
             self.delUserError( info[0], info[1] )
         elif exc.code in [205, 209, 217, 219, 224, 231, 500]:
             self.protoError( exc.code )
+	#fix crash when adding invalid usernames
+	elif exc.code == 208:
+	    self.pError( _('The user may not be added because no account exists by that name.') )
         else:
             self.fatalProtoError( exc.code )
 
@@ -883,11 +896,11 @@ class Session( Comm ):
                         chat.ready= 1
                         if elem[1] == 'NORMAL':
                             self.inputs.add( chat.s, SOCK_CHAT )
-                            chat.send( 'USR', '%s %s\n' % (self.user, chat.hash) )
+                            chat.send( 'USR', '%s %s\r\n' % (self.user, chat.hash) )
                             self.pDebug( 'USR sent' )
                         else:
                             #self.s.setblocking( 0 )
-                            chat.send( 'ANS', '%s %s %s\n' % (self.user
+                            chat.send( 'ANS', '%s %s %s\r\n' % (self.user
                                     , chat.hash, chat.sessionId) )
                         chat.state= CHAT_WAITING_REPLY            
                         #self.delConnectingChats( chat )
@@ -1017,9 +1030,9 @@ class Session( Comm ):
                     self.listShown= 1
 
             elif ( msg.type == 'VER' ):
-                self.send( 'CVR', '0x0409 winnt 5.1 i386 MSNMSGR 5.0.0540 MSMSGS %s\n' % self.user )
+                self.send( 'CVR', '0x0409 winnt 5.1 i386 MSNMSGR 5.0.0540 MSMSGS %s\r\n' % self.user )
             elif ( msg.type == 'INF' ):
-                self.send( 'USR', 'MD5 I %s\n' % self.user )
+                self.send( 'USR', 'MD5 I %s\r\n' % self.user )
             elif ( msg.type == 'ILN' ):
                 name= unicode( urllib.unquote(msg.fields['name']), 'utf-8'
                         , 'replace' )
@@ -1028,7 +1041,7 @@ class Session( Comm ):
                     'name': name , 'state':msg.fields['state']}
                 self.userOn( msg.fields['passport'] )
             elif ( msg.type == 'CVR' ):
-                self.send( 'USR', 'TWN I %s\n' % self.user )
+                self.send( 'USR', 'TWN I %s\r\n' % self.user )
             elif ( msg.type == 'NLN' ):
                 name= unicode( urllib.unquote(msg.fields['name']), 'utf-8'
                         , 'replace' )
@@ -1066,7 +1079,7 @@ class Session( Comm ):
                     #self.pDebug( 'Socket: ' + str(self.s) )
                     #self.pDebug( 'Inputs: ' + str(self.inputs.fileDescs) )
                     #self.send( 'USR', 'MD5 I %s\r\n' % self.user )
-                    self.send( 'VER', 'MSNP8 CVRO\n' )
+                    self.send( 'VER', 'MSNP8 CVRO\r\n' )
                 else: # Per chat server (Switchboard server)
                     c= self.pendingChats[msg.fields['id']] 
                     del( self.pendingChats[msg.fields['id']] )
@@ -1087,14 +1100,14 @@ class Session( Comm ):
                 if msg.fields['type'] == 'MD5':
                     hash= msg.fields['arg2']
                     chunk= md5.new( hash + self.passwd ).hexdigest()
-                    self.send( 'USR', 'MD5 S %s\n' % chunk )
+                    self.send( 'USR', 'MD5 S %s\r\n' % chunk )
                 elif msg.fields['type'] == 'TWN': 
                     text= split(msg.st)[4]
                     #text= urllib.unquote( text )
                     #self.pageParams= replace( text, ',', '&' )
                     self.pDebug( 'PARAMS: ' +  text )
                     param= self.sslLogin( text )
-                    self.send( 'USR', 'TWN S %s\n' % param )
+                    self.send( 'USR', 'TWN S %s\r\n' % param )
                 else:    
                     name= unicode( urllib.unquote(msg.fields['arg2']), 'utf-8'
                             , 'replace' )
@@ -1162,9 +1175,16 @@ class Session( Comm ):
                     #self.listUpdated( self.requestedList, msg.fields['list'] )
 
                 
+            elif ( msg.type == 'NOT' ): # Notification (MSN Mobile, alert...)
+                more= self.inputs.getChars( res[0], int(msg.fields['size']) )
+
+                # there's only part of notification, return what we read 
+                # (NOT line)
+                if not more:
+                    self.inputs.strReturn( res[0], msg.st + '\r\n' )
+
             elif ( msg.type == 'MSG' ): # Mail notification
-                more= self.inputs.getChars( res[0]
-                        , int(msg.fields['arg3']) )
+                more= self.inputs.getChars( res[0], int(msg.fields['arg3']) )
 
                 # there's only part of msg, return what we read (MSG line)
                 if not more:
@@ -1190,7 +1210,10 @@ class Session( Comm ):
             if ( msg.type == 'BYE' ):
                 if self.chats.has_key( res[0] ):
                     chat= self.chats[res[0]]
-                    chat.delUser( msg.fields['arg'] )
+                    if msg.fields.has_key('idleclose'):
+                        chat.delUser( msg.fields['arg'], msg.fields['idleclose'] )
+                    else:
+                        chat.delUser( msg.fields['arg'] )
                     if len(chat.users) == 0 and CLOSE_EMPTY:
                         self.pDebug( 'chat borrado!' )
                         self.inputs.remove( res[0] )
@@ -1539,6 +1562,7 @@ class Chat( Comm ):
                 self.logText( LOG_CHAT_INIT % (user, time.asctime()) )
         
         self.users[user]= ''
+        self.hideSocketError= 1
 
     def nextNotActive( self ):
         for f in self.users.keys():
@@ -1558,8 +1582,14 @@ class Chat( Comm ):
     def isTyping( self, user ):    
         return self.users[user] == 'TYPING'
 
-    def delUser( self, user ):
+    # idleClose is 1 when server closed socket when idle after 5 min.
+    def delUser( self, user, idleClose=0 ):
         del( self.users[user] )
+        self.pDebug('self.delUser: ' + str(self.listUsers()))
+        if len(self.listUsers()) == 0 or idleClose:
+            self.hideSocketError= 1
+        else:    
+            self.hideSocketError= 0
    
     def getUsers( self ):
         return self.users.items()
@@ -1625,7 +1655,7 @@ class Chat( Comm ):
         fSend.init()
 
     def invite( self, passport ):
-        self.send( 'CAL', '%s\n' % (passport) )
+        self.send( 'CAL', '%s\r\n' % (passport) )
 
     def accept( self, msg ):
         self.user= msg.fields['passport']
@@ -1662,7 +1692,9 @@ class Chat( Comm ):
             
 
     def typingNotification( self ):
-        if self.ready:
+        self.pDebug('typingNotification')
+        self.pDebug(self.ready)
+        if self.numOnlineUsers() > 0:
             chunk= TYPING_HEADER + self.session.user + '\r\n\r\n\r\n'
             self.send( 'MSG', 'U %d\r\n%s' % ( len( chunk ), chunk ) )
 
@@ -1703,6 +1735,14 @@ class Chat( Comm ):
                 self.session.isTyping= 0
 
         self.actions=[]
+
+    def numOnlineUsers( self ):
+        n = 0
+        for state in self.users.values():
+            if state == 'ONLINE':
+                n += 1
+    
+        return n
     
 ###############################################################################
 
